/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.visualweb.designer.cssengine;

/**
 * This class provides a manager for the margin-top, margin-left,
 * margin-right, and margin-bottom properties.
 *
 * It's very similar to PaddingManager, except margins can be 'auto'
 * and paddings can not.
 *
 * @author Tor Norbye
 */
public class MarginManager extends AutoableLengthManager {
    private String property;

    public MarginManager(String property) {
        this.property = property;
    }

    public boolean isInheritedProperty() {
        return false;
    }

    public String getPropertyName() {
        return property;
    }

    protected int getOrientation() {
        // NOTE: BOTH horizontal and vertical margins are relative
        // to the WIDTH of the containing block! So in particular,
        // margin-top: 50% is UNRELATED to the height of the containing
        // block! Therefore, I always return a horizontal orientation
        // so we compute relative to the block width.
        // This is specified in the CSS2 spec section 8.3.
        return HORIZONTAL_ORIENTATION;
    }
}
